@@ -63,7 +63,7 @@ class Curve(ABC):
         pass
 
     @abstractmethod
-    def force_continuity(self, next_point: Vector2) -> Vector2:
+    def force_continuity(self, next_point: Vector2, close_circuit: bool = False) -> Vector2:
         pass
 
     @classmethod
@@ -109,14 +109,14 @@ class Arc(Curve):
         return curves, []
 
     @classmethod
-    def from_arcbox(cls, id:str, orientation: Orientation, origin: Vector2, arc_box_size: Vector2):
+    def from_arcbox(cls, id: str, orientation: Orientation, origin: Vector2, arc_box_size: Vector2):
         return Arc(id, orientation, origin, arc_box_size)
 
     @classmethod
-    def from_origin_and_destination(cls, id:str, orientation: Orientation, origin: Vector2, destination: Vector2):
+    def from_origin_and_destination(cls, id: str, orientation: Orientation, origin: Vector2, destination: Vector2):
         return Arc(id, orientation, origin, Vector2(destination.x - origin.x, destination.y - origin.y) * 2)
 
-    def __init__(self, id:str, orientation: Orientation, origin: Vector2, arc_box_size: Vector2):
+    def __init__(self, id: str, orientation: Orientation, origin: Vector2, arc_box_size: Vector2):
         super().__init__(id, origin)
         self.orientation = orientation
         self.arc_box_size = arc_box_size
@@ -161,7 +161,7 @@ class Arc(Curve):
             raise Exception(f"Invalid curve orientation: [{self.orientation}]")
 
     def _end_point(self):
-        if self.orientation in (Orientation.DOWN_LEFT,  Orientation.LEFT_DOWN):
+        if self.orientation in (Orientation.DOWN_LEFT, Orientation.LEFT_DOWN):
             return Vector2(int(self.origin.x - self.arc_box_size.x / 2), int(self.origin.y + self.arc_box_size.y / 2))
         elif self.orientation in (Orientation.LEFT_UP, Orientation.UP_LEFT):
             return Vector2(int(self.origin.x - self.arc_box_size.x / 2), int(self.origin.y - self.arc_box_size.y / 2))
@@ -196,7 +196,8 @@ class Arc(Curve):
         ret = []
         increment = (self.arc_rad_end - self.arc_rad_start) / (numPoints - 1)
         for ii in range(0, numPoints):
-            next = self.point_along_arc(self.arc_rad_start + increment * ii, self.mid_point, self.arc_box_as_rectangle())
+            next = self.point_along_arc(self.arc_rad_start + increment * ii, self.mid_point,
+                                        self.arc_box_as_rectangle())
             ret.append(next)
 
         return ret
@@ -244,7 +245,10 @@ class Arc(Curve):
     def length(self):
         return (self.arc_rad_end - self.arc_rad_start) * (self.origin - self.mid_point).length()
 
-    def force_continuity(self, next_point: Vector2) -> Vector2:
+    def force_continuity(self, next_point: Vector2, close_circuit: bool = False) -> Vector2:
+        if close_circuit:
+            raise NotImplementedError(f"Close circuit not implemented for type [{type(self)}]")
+
         end_point = self.end_point()
         if self.orientation in (Orientation.UP_LEFT, Orientation.DOWN_LEFT) and next_point.x > end_point.x:
             return Vector2(end_point.x, next_point.y)
@@ -257,6 +261,7 @@ class Arc(Curve):
         else:
             return next_point
 
+
 class CircularArc(Curve):
     @classmethod
     def from_points(cls, points: List[Vector2], naming_provider: Callable[[], str]) -> (List, List):
@@ -274,8 +279,8 @@ class CircularArc(Curve):
         for ii in range(0, len(used_points) - 3, 3):
             curves.append(CircularArc(naming_provider(),
                                       origin=points[ii],
-                                      destination=points[ii+1],
-                                      center=points[ii+2]))
+                                      destination=points[ii + 1],
+                                      center=points[ii + 2]))
 
         return curves, leftover_points
 
@@ -300,9 +305,7 @@ class CircularArc(Curve):
 
         return ret
 
-
-
-    def __init__(self, id: str, origin:Vector2, destination:Vector2, center: Vector2):
+    def __init__(self, id: str, origin: Vector2, destination: Vector2, center: Vector2):
         """ Create a circle given a center, start, and end. The arc is assumed to be the counter-clockwise movement from origin to
         destination, so enter the o and d appropriately based on which segment between points is required"""
 
@@ -350,7 +353,6 @@ class CircularArc(Curve):
     def destination_rads(self):
         return self.circle.rads_of_point(self.destination)
 
-
     def _end_point(self):
         return self.destination
 
@@ -360,10 +362,10 @@ class CircularArc(Curve):
     def line_representation(self) -> List[Line]:
         pass
 
-    def force_continuity(self, next_point: Vector2) -> Vector2:
-        pass
+    def force_continuity(self, next_point: Vector2, close_circuit: bool = False) -> Vector2:
+        raise NotImplementedError(f"force_circuit not implemented for type [{type(self)}]")
 
-    def compute_circulararc_points(self, numPoints: int=None):
+    def compute_circulararc_points(self, numPoints: int = None):
         if numPoints is None:
             numPoints = 30
 
@@ -379,13 +381,13 @@ class CircularArc(Curve):
 
 class LineCurve(Curve):
     @classmethod
-    def from_points(self, points, naming_provider: Callable [[], str]) -> (List, List):
+    def from_points(self, points, naming_provider: Callable[[], str]) -> (List, List):
         lines = []
         for ii in range(len(points) - 1):
             lines.append(LineCurve(naming_provider(), points[ii], points[ii + 1]))
         return lines, []
 
-    def __init__(self, id:str, origin: Vector2, destination: Vector2):
+    def __init__(self, id: str, origin: Vector2, destination: Vector2):
         super().__init__(id, origin)
         self.destination = destination
         self.length = self.length()
@@ -402,12 +404,15 @@ class LineCurve(Curve):
     def length(self):
         return Line(self.origin, self.destination).length
 
-    def force_continuity(self, next_point: Vector2) -> Vector2:
+    def force_continuity(self, next_point: Vector2, close_circuit: bool = False) -> Vector2:
+        if close_circuit:
+            raise NotImplementedError(f"Close circuit not implemented for type [{type(self)}]")
         return next_point
 
+
 class CubicBezier(Curve):
     @classmethod
-    def as_circular_arcs(cls, curve, tolerance:float = 10):
+    def as_circular_arcs(cls, curve, tolerance: float = 10):
         if not type(curve) == CubicBezier:
             raise TypeError(f"provided value was not of type {CubicBezier}. {type(curve)} provided")
 
@@ -429,16 +434,15 @@ class CubicBezier(Curve):
                 return [CircularArc.from_3_consecutive_points(a=circle.known_boundary_points[0],
                                                               b=circle.known_boundary_points[1],
                                                               c=circle.known_boundary_points[2],
-                                                              naming_provider=lambda : str(uuid4()))]
+                                                              naming_provider=lambda: str(uuid4()))]
 
         # handle subdivide and try again
         sub_divisions = curve.sub_divide_at_t(0.5)
         arcs = [CubicBezier.as_circular_arcs(x, tolerance) for x in sub_divisions]
         return [item for sublist in arcs for item in sublist]
 
-
     @classmethod
-    def from_points(cls, points: List[Vector2], naming_provider: Callable [[], str]) -> (List, List):
+    def from_points(cls, points: List[Vector2], naming_provider: Callable[[], str]) -> (List, List):
         leftover_points = []
         leftover_count = (len(points) - 1) % 3 if len(points) >= 4 else len(points)
         leftover_points = points[-leftover_count:] if leftover_count != 0 else leftover_points
@@ -451,11 +455,11 @@ class CubicBezier(Curve):
 
         # A Cubic Bezier requires 4 control points, so operate in blocks of 4 (re-using the last each time)
         for ii in range(0, len(used_points) - 3, 3):
-            curves.append(CubicBezier(naming_provider(), points[ii:ii+4]))
+            curves.append(CubicBezier(naming_provider(), points[ii:ii + 4]))
 
         return curves, leftover_points
 
-    def __init__(self, id:str, control_points: List[Vector2]):
+    def __init__(self, id: str, control_points: List[Vector2]):
 
         if all(point.distance_from(control_points[0]) < .001 for point in control_points):
             raise ValueError(f"the provided control points are invalid as they are all the same: {control_points[0]}")
@@ -463,8 +467,6 @@ class CubicBezier(Curve):
         Curve.__init__(self, id, control_points[0])
         self._cps = control_points
 
-
-
     def line_representation(self):
         return self.compute_curve_lines()
 
@@ -550,11 +552,12 @@ class CubicBezier(Curve):
 
         return result
 
+    def force_continuity(self, next_point: Vector2, close_circuit: bool = False) -> Vector2:
 
-    def force_continuity(self, next_point: Vector2) -> Vector2:
-        return next_point.project_onto(self.control_points[-1], self.control_points[-2])
-
-
+        if close_circuit:
+            return next_point.project_onto(self.control_points[0], self.control_points[1])
+        else:
+            return next_point.project_onto(self.control_points[-1], self.control_points[-2])
 
     def biarc_circle(self) -> (Circle, float):
         p1 = self.control_points[0]
@@ -586,10 +589,6 @@ class CubicBezier(Curve):
 
         return biarc_circ, max_delta
 
-
-
-
-
     def sub_divide_at_t(self, t: float):
         if not (0 < t < 1):
             raise ValueError(f"t must be between 0 and 1 but {t} was provided")
@@ -608,8 +607,6 @@ class CubicBezier(Curve):
         return [CubicBezier(id=str(uuid4()), control_points=[p1, r2, r3, point_t]),
                 CubicBezier(id=str(uuid4()), control_points=[point_t, s2, s3, p4])]
 
-
-
     def point_at_t(self, t: float):
         if not (0 < t < 1):
             raise ValueError(f"t must be between 0 and 1 but {t} was provided")
@@ -656,7 +653,7 @@ class CubicBezier(Curve):
         return ret
 
     def inflection_points(self):
-        #https://stackoverflow.com/questions/35901079/calculating-the-inflection-point-of-a-cubic-bezier-curve
+        # https://stackoverflow.com/questions/35901079/calculating-the-inflection-point-of-a-cubic-bezier-curve
 
         b0x = self.control_points[0].x
         b0y = self.control_points[0].y
@@ -712,12 +709,12 @@ class CatmullRom(Curve):
         curves.append(CatmullRom(naming_provider(), points))
         return curves, leftover_points
 
-
-    def __init__(self, id:str, control_points: List[Vector2]):
+    def __init__(self, id: str, control_points: List[Vector2]):
         Curve.__init__(self, id, control_points[0])
 
         if len(control_points) < 4:
-            raise ValueError(f"Invalid input for control points. Must have at least 4 values but list of length {len(control_points)} was provided: [{control_points}]")
+            raise ValueError(
+                f"Invalid input for control points. Must have at least 4 values but list of length {len(control_points)} was provided: [{control_points}]")
         self._cps = control_points
 
     def _control_points(self) -> List[Vector2]:
@@ -795,23 +792,25 @@ class CatmullRom(Curve):
 
             c.extend(C)
 
-
         return [Vector2(point[0], point[1]) for point in c]
 
-
-    def force_continuity(self, next_point: Vector2) -> Vector2:
+    def force_continuity(self, next_point: Vector2, close_circuit: bool = False) -> Vector2:
+        if close_circuit:
+            raise NotImplementedError(f"Close circuit not implemented for type [{type(self)}]")
         return next_point
 
+
 class CurveType(Enum):
     ARC = 1,
     CUBICBEZIER = 2,
     LINE = 3,
     CATMULLROM = 4
 
+
 class CurveBuilder:
 
-    def __init__(self, id_provider: Callable [[], str]):
-        self.curves = {} # {id: curve}
+    def __init__(self, id_provider: Callable[[], str]):
+        self.curves = {}  # {id: curve}
         self.current_points = []
         self.current_curve_type = EnumToggleable(CurveType)
         self.current_orientation = EnumToggleable(Orientation)
@@ -829,12 +828,20 @@ class CurveBuilder:
 
     def save_curve(self):
         new_curves, _ = self.curves_from_points(self.current_curve_type.value, self.current_points)
+
+        new_curve_dict = {}
         for curve in new_curves:
-            self.curves[self.id_provider()] = curve
+            new_id = self.id_provider()
+            new_curve_dict[new_id] = curve
+            self.curves[new_id] = curve
         self.current_points = []
 
+        return new_curve_dict
+
     def add_point(self, point: Vector2):
-        self.current_points.append(point)
+        if len(self.current_points) == 0 or\
+                (point != self.current_points[-1]):
+            self.current_points.append(point)
 
     def remove_point(self):
         if any(self.current_points):
@@ -843,7 +850,7 @@ class CurveBuilder:
     def curves_from_points(self, curve_type: CurveType, points: List[Vector2]) -> (List[Curve], List[Vector2]):
         # curves = []
         # leftover_points = []
-        if len(points) <2:
+        if len(points) < 2:
             return [], points
 
         if curve_type == CurveType.LINE:
@@ -873,13 +880,12 @@ class CurveBuilder:
         all_points = self.current_points + next_points
         points = []
         for i in all_points:
-            if i not in points or i == points[0]:
-                points.append(i)
+            points.append(i)
 
         curves, leftovers = self.curves_from_points(self.current_curve_type.value, points)
         return curves
 
-    def force_continuity(self, next_point: Vector2):
+    def force_continuity(self, next_point: Vector2, close_circuit: bool = False):
         curves, leftovers = self.curves_from_points(self.current_curve_type.value, self.current_points)
 
         if curves is None or len(curves) == 0:
@@ -888,6 +894,8 @@ class CurveBuilder:
         # Dont need to force a continuity if we are in the middle of a Bezier
         if self.current_curve_type.value == CurveType.CUBICBEZIER and len(leftovers) != 0:
             return next_point
+        elif close_circuit:
+            return curves[0].force_continuity(next_point, close_circuit)
         else:
             return curves[-1].force_continuity(next_point)
 
