@@ -12,6 +12,10 @@ class Rectangle:
     def points_tuple(self):
         return ((self.x, self.y), (self.x + self.width, self.y), (self.x + self.width, self.y + self.height), (self.x, self.y + self.height))
 
+    @property
+    def center(self) -> Vector2:
+        return Vector2(self.x + self.width / 2, self.y + self.height / 2)
+
     def __str__(self):
         return f"TopLeft: <{self.x}, {self.y}>, Size: H{self.height} x W{self.width}"
 
@@ -146,3 +150,8 @@ class Triangle:
         len_ca = (self.c - self.a).length()
 
         return (len_bc * self.a + len_ab * self.c + len_ca * self.b) / (len_bc + len_ca + len_ab)
+
+if __name__ == "__main__":
+    rect = Rectangle(100, 10, 25, 50)
+
+    print(rect.center)
\ No newline at end of file
